searchData={"content_type":"text/markdown","items":[{"doc":"The main module implements Douglas Crockford's [Base32](https://www.crockford.com/base32.html) encoding.","ref":"CrockfordBase32.html","title":"CrockfordBase32","type":"module"},{"doc":"Decode the encoded to a bitstring, all hyphen(s) are removed and ignore the encoded's case.\n\nIf the encoded bitstring be with a check symbol, require to use `checksum: true` in decoding.","ref":"CrockfordBase32.html#decode_to_bitstring/2","title":"CrockfordBase32.decode_to_bitstring/2","type":"function"},{"doc":"iex> CrockfordBase32.decode_to_bitstring(\"C5H66\")\n    {:ok, \"abc\"}\n    iex> CrockfordBase32.decode_to_bitstring(\"C5H66C\", checksum: true)\n    {:ok, \"abc\"}\n    iex> CrockfordBase32.decode_to_bitstring(\"C5H66D\", checksum: true)\n    :error_checksum","ref":"CrockfordBase32.html#decode_to_bitstring/2-example","title":"Example - CrockfordBase32.decode_to_bitstring/2","type":"function"},{"doc":"The same to the options of `decode_to_integer/2`.","ref":"CrockfordBase32.html#decode_to_bitstring/2-options","title":"Options - CrockfordBase32.decode_to_bitstring/2","type":"function"},{"doc":"Decode the encoded string to an integer, all hyphen(s) are removed and ignore the encoded's case.\n\nIf the encoded string be with a check symbol, require to use `checksum: true` in decoding.","ref":"CrockfordBase32.html#decode_to_integer/2","title":"CrockfordBase32.decode_to_integer/2","type":"function"},{"doc":"iex> CrockfordBase32.decode_to_integer(\"16JD\", checksum: true)\n    {:ok, 1234}\n    iex> CrockfordBase32.decode_to_integer(\"16j\")\n    {:ok, 1234}\n    iex> CrockfordBase32.decode_to_integer(\"16j*\", checksum: true)\n    :error_checksum","ref":"CrockfordBase32.html#decode_to_integer/2-example","title":"Example - CrockfordBase32.decode_to_integer/2","type":"function"},{"doc":"* `:checksum`, optional, a boolean, by defaults to `false` means expect input the encoded string without a check symbol in its tail,\n    if set it as `true`, please ensure input encoded is a string be with a check symbol, or return  `:error_checksum`.","ref":"CrockfordBase32.html#decode_to_integer/2-options","title":"Options - CrockfordBase32.decode_to_integer/2","type":"function"},{"doc":"Encode an integer or a bitstring in Crockford's Base32.\n\nAfter encoded, the return string only contains these characters set(`\"0123456789ABCDEFGHJKMNPQRSTVWXYZ\"`, 10 digits and\n22 letters, excluding `\"I\"`, `\"L\"`, `\"O\"` and `\"U\"`), if set `checksum: true`, there would be with one of these check \nsymbols(`\"*~$=U\"`) or the previous 10 digits and 22 letters as the last character.","ref":"CrockfordBase32.html#encode/2","title":"CrockfordBase32.encode/2","type":"function"},{"doc":"iex> CrockfordBase32.encode(1234567)\n    \"15NM7\"\n    iex> CrockfordBase32.encode(1234567, checksum: true)\n    \"15NM7S\"\n    iex> CrockfordBase32.encode(1234567, split_size: 3)\n    \"15N-M7\"\n    iex> CrockfordBase32.encode(1234567, split_size: 3, checksum: true)\n    \"15N-M7S\"","ref":"CrockfordBase32.html#encode/2-example","title":"Example - CrockfordBase32.encode/2","type":"function"},{"doc":"* `:checksum`, optional, a boolean, by defaults to `false`, if set it as `true` will calculate a check\n    symbol and append it to the return string.\n  * `:split_size`, optional, a positive integer, if set it will use it as a step size to split\n    the return string with hyphen(s).","ref":"CrockfordBase32.html#encode/2-options","title":"Options - CrockfordBase32.encode/2","type":"function"},{"doc":"# CrockfordBase32\n\nAn Elixir Implementation of Douglas Crockford's Base32 encoding.\n\nPlease see [https://www.crockford.com/base32.html](https://www.crockford.com/base32.html).\n\nThis library can encode an integer or a bitstring in Crockford's Base32, and also provide the way to decode the corresponding encoding.","ref":"readme.html","title":"CrockfordBase32","type":"extras"},{"doc":"```elixir\ndef deps do\n  [\n    {:crockford_base32, \"~> 0.7\"}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - CrockfordBase32","type":"extras"},{"doc":"","ref":"readme.html#usage","title":"Usage - CrockfordBase32","type":"extras"},{"doc":"Encode an integer:\n\n```elixir\niex> CrockfordBase32.encode(1234)\n\"16J\"\n```\n\nEncode an integer with `checksum: true`:\n\n```elixir\niex> CrockfordBase32.encode(1234, checksum: true)\n\"16JD\"\n```\n\nEncode an inetger, and insert hyphens (-) per the step size(via `split_size`) in encoded result:\n\n```elixir\niex> CrockfordBase32.encode(1234, split_size: 2)\n\"16-J\"\niex> CrockfordBase32.encode(1234, split_size: 1)\n\"1-6-J\"\niex> CrockfordBase32.encode(1234, split_size: 1, checksum: true)\n\"1-6-J-D\"\n```\n\nEncode a bitstring, and optional `split_size` and `checksum` options are both working:\n\n```elixir\niex> CrockfordBase32.encode(<<12345678::size(48)>>)\n\"00001F319R\"\niex> CrockfordBase32.encode(\"abc\")\n\"C5H66\"\niex> CrockfordBase32.encode(\"abc\", checksum: true)\n\"C5H66C\"\niex> CrockfordBase32.encode(\"abc\", checksum: true, split_size: 3)\n\"C5H-66C\"\niex> CrockfordBase32.encode(<<5::size(3)>>)\n\"M\"\n```","ref":"readme.html#encode","title":"Encode - CrockfordBase32","type":"extras"},{"doc":"There will internally remove all hyphen(s) before decoding.\n\nDecode the encoded to an integer:\n\n```elixir\niex> CrockfordBase32.decode_to_integer(\"16J\")\n{:ok, 1234}\niex> CrockfordBase32.decode_to_integer(\"16-J\")\n{:ok, 1234}\niex> CrockfordBase32.decode_to_integer(\"16-j\")\n{:ok, 1234}\n```\n\nWith a check symbol, and decode the encoded to an integer:\n\n```elixir\niex> CrockfordBase32.decode_to_integer(\"16JD\", checksum: true)\n{:ok, 1234}\niex> CrockfordBase32.decode_to_integer(\"16J1\", checksum: true)\n:error_checksum\n```\n\nDecode the encoded to a bitstring:\n\n```elixir\niex> CrockfordBase32.decode_to_bitstring(\"00001F319R\")\n{:ok, <<0, 0, 0, 188, 97, 78>>}\niex> CrockfordBase32.decode_to_bitstring(\"C5H66\")\n{:ok, \"abc\"}\niex> CrockfordBase32.decode_to_bitstring(\"C5H-66\")\n{:ok, \"abc\"}\niex> CrockfordBase32.decode_to_bitstring(\"c5H-66\")\n{:ok, \"abc\"}\niex> CrockfordBase32.decode_to_bitstring(\"c5h-66\")\n{:ok, \"abc\"}\niex> CrockfordBase32.decode_to_bitstring(\"c5h66\")\n{:ok, \"abc\"}\niex> CrockfordBase32.decode_to_bitstring(\"M\")\n{:ok, <<5::size(3)>>}\n```\n\nWith a check symbol, and decode the encoded to a bitstring:\n\n```elixir\niex> CrockfordBase32.decode_to_bitstring(\"C5H66C\", checksum: true)\n{:ok, \"abc\"}\niex> CrockfordBase32.decode_to_bitstring(\"C5H66D\", checksum: true)\n:error_checksum\n```\n\nSome invalid cases:\n\n```elixir\niex> CrockfordBase32.decode_to_bitstring(<<1, 2, 3>>)\n:error\niex> CrockfordBase32.decode_to_bitstring(<<>>)\n:error\niex> CrockfordBase32.decode_to_integer(<<1, 2, 3>>)\n:error\niex> CrockfordBase32.decode_to_integer(<<>>)\n:error\n```","ref":"readme.html#decode","title":"Decode - CrockfordBase32","type":"extras"},{"doc":"In some cases, you may want to encode the fixed size bytes, we can do this be with a better performance leverages the benefit of the pattern match of Elixir/Erlang. I use this feature to implement a [ULID](https://github.com/xinz/elixir_ulid) in Elixir.\n\nRefer [ULID specification](https://github.com/ulid/spec#specification), a ULID concatenates a UNIX timestamp in milliseconds(a 48 bit integer) and a randomness in 80-bit, since an integer in bits are padded with some `<<0::1>>` leading when needed, and a ULID in 128-bit after encoded its length is 26 (can be divisible by 5), apply the fixed size encoding with `type: :integer` can efficiently encode/decode a ULID, for example:\n\n```elixir\ndefmoule ULID do\n\n  defmoule Base32.Bits128 do\n    use CrockfordBase32,\n      bits_size: 128,\n      type: :integer # Optional, defaults to `:bitstring`\n  end\n\nend\n```\n\nThen we can use `ULID.Base32.Bits128` to encode/decode a 128-bit bitstring which concatenates an integer (as UNIX timestamp in millisecond) in 48-bit and a randomly generated in 80-bit.\n\n#### Padding 0-bit when decoding\n\nCrockford's Base32 avoids the use of padding characters by zero-extending the data to ensure the bit-length is a multiple of 5, there is no need to retain additional padding bits(`<<0::size(1)>>`) in the decoded result, so there may some decoded bits that are not as complete as expected, for example:\n\nA string(`\"01HY3B3HQ5FMEVJN8ME7C4HZDM\"`) is a 26 length randomly generated string as a suffix of [TypeID](https://github.com/jetify-com/typeid), TypeID's specification defines its suffix base32 encoding be with two zeroed bits are pre-pended to the 128-bits of the UUID, resulting in 130-bits of data.\n\nNotice: Please ignore case in the following parameter `\"s\"`, CrockfordBase32 is not case sensitive, but TypeID only uses lowercase.\n\n```elixir\niex> s = \"01HY3B3HQ5FMEVJN8ME7C4HZDM\"\niex> {:ok, input} = CrockfordBase32.decode_to_bitstring(s)\n{:ok, <<0, 99, 225, 172, 113, 185, 95, 71, 110, 85, 69, 28, 118, 18, 63, 109>>}\niex> bit_size(input)\n128\niex> CrockfordBase32.encode(< >)\n\"01HY3B3HQ5FMEVJN8ME7C4HZDM\"\niex> <<0::size(2), uuid::bitstring>> = < >\n<<0, 99, 225, 172, 113, 185, 95, 71, 110, 85, 69, 28, 118, 18, 63, 109,\n  0::size(2)>>\niex> uuid\n<<1, 143, 134, 177, 198, 229, 125, 29, 185, 85, 20, 113, 216, 72, 253, 180>>\n```\n\nWe must explicitly append two zero bits(`<<0::size(2)>>`) into the `<<0, 99, 225, 172, 113, 185, 95, 71, 110, 85, 69, 28, 118, 18, 63, 109>>` and only later we can get the correct `uuid` bitstring.\n\nSimilar to TypeID, pre-defining a fixed-size bit string encoding, we can do this:\n\n```elixir\ndefmoule Typeid do\n\n  defmoule Base32.Bits130 do\n    use CrockfordBase32,\n      bits_size: 130\n  end\n\nend\n```\n\nUse `\"Typeid.Base32.Bits130\"` and then do not need to manually to pad the zero bit(s), it will use its fixed size to handle the padding.\n\n```\niex> Typeid.Base32.Bits130.decode(\"01HY3B3HQ5FMEVJN8ME7C4HZDM\")\n{:ok,\n <<0, 99, 225, 172, 113, 185, 95, 71, 110, 85, 69, 28, 118, 18, 63, 109,\n   0::size(2)>>}\niex> {:ok, input} = Typeid.Base32.Bits130.decode(\"01HY3B3HQ5FMEVJN8ME7C4HZDM\")\n{:ok,\n <<0, 99, 225, 172, 113, 185, 95, 71, 110, 85, 69, 28, 118, 18, 63, 109,\n   0::size(2)>>}\niex> bit_size(input)\n130\niex> <<0::size(2), uuid::bitstring>> = input\n<<0, 99, 225, 172, 113, 185, 95, 71, 110, 85, 69, 28, 118, 18, 63, 109,\n  0::size(2)>>\niex> uuid\n<<1, 143, 134, 177, 198, 229, 125, 29, 185, 85, 20, 113, 216, 72, 253, 180>>\niex> Typeid.Base32.Bits130.encode(input)\n\"01HY3B3HQ5FMEVJN8ME7C4HZDM\"\n```","ref":"readme.html#fixed-size-encoding","title":"Fixed Size Encoding - CrockfordBase32","type":"extras"},{"doc":"These libraries or tools are very helpful in understanding and reference, thanks!\n\n- [TheRealReal/ecto-ulid](https://github.com/TheRealReal/ecto-ulid)\n- [shiguredo/base32_clockwork](https://github.com/shiguredo/base32_clockwork)\n- [voldy/base32_crockford](https://github.com/voldy/base32_crockford)\n- [levinalex/base32](https://github.com/levinalex/base32)\n- [jbittel/base32-crockford](https://github.com/jbittel/base32-crockford)\n- [dcode.fr's crockford-base32 encoding](https://www.dcode.fr/crockford-base-32-encoding)","ref":"readme.html#credits","title":"Credits - CrockfordBase32","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,51,50,46,50]}}